# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.loyalty_event_query import LoyaltyEventQueryParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.search_loyalty_events_response import SearchLoyaltyEventsResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLoyaltyClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search_events(
        self,
        *,
        query: typing.Optional[LoyaltyEventQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchLoyaltyEventsResponse]:
        """
        Searches for loyalty events.

        A Square loyalty program maintains a ledger of events that occur during the lifetime of a
        buyer's loyalty account. Each change in the point balance
        (for example, points earned, points redeemed, and points expired) is
        recorded in the ledger. Using this endpoint, you can search the ledger for events.

        Search results are sorted by `created_at` in descending order.

        Parameters
        ----------
        query : typing.Optional[LoyaltyEventQueryParams]
            A set of one or more predefined query filters to apply when
            searching for loyalty events. The endpoint performs a logical AND to
            evaluate multiple filters and performs a logical OR on arrays
            that specifies multiple field values.

        limit : typing.Optional[int]
            The maximum number of results to include in the response.
            The last page might contain fewer events.
            The default is 30 events.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchLoyaltyEventsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/loyalty/events/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=LoyaltyEventQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchLoyaltyEventsResponse,
                    construct_type(
                        type_=SearchLoyaltyEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawLoyaltyClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search_events(
        self,
        *,
        query: typing.Optional[LoyaltyEventQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchLoyaltyEventsResponse]:
        """
        Searches for loyalty events.

        A Square loyalty program maintains a ledger of events that occur during the lifetime of a
        buyer's loyalty account. Each change in the point balance
        (for example, points earned, points redeemed, and points expired) is
        recorded in the ledger. Using this endpoint, you can search the ledger for events.

        Search results are sorted by `created_at` in descending order.

        Parameters
        ----------
        query : typing.Optional[LoyaltyEventQueryParams]
            A set of one or more predefined query filters to apply when
            searching for loyalty events. The endpoint performs a logical AND to
            evaluate multiple filters and performs a logical OR on arrays
            that specifies multiple field values.

        limit : typing.Optional[int]
            The maximum number of results to include in the response.
            The last page might contain fewer events.
            The default is 30 events.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchLoyaltyEventsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/loyalty/events/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=LoyaltyEventQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchLoyaltyEventsResponse,
                    construct_type(
                        type_=SearchLoyaltyEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .search_vendors_request_sort_field import SearchVendorsRequestSortField
import pydantic
from .sort_order import SortOrder
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchVendorsRequestSort(UncheckedBaseModel):
    """
    Defines a sorter used to sort results from [SearchVendors](api-endpoint:Vendors-SearchVendors).
    """

    field: typing.Optional[SearchVendorsRequestSortField] = pydantic.Field(default=None)
    """
    Specifies the sort key to sort the returned vendors.
    See [Field](#type-field) for possible values
    """

    order: typing.Optional[SortOrder] = pydantic.Field(default=None)
    """
    Specifies the sort order for the returned vendors.
    See [SortOrder](#type-sortorder) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .upsert_order_custom_attribute_response import UpsertOrderCustomAttributeResponseParams


class BulkUpsertOrderCustomAttributesResponseParams(typing_extensions.TypedDict):
    """
    Represents a response from a bulk upsert of order custom attributes.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    values: typing.Dict[str, UpsertOrderCustomAttributeResponseParams]
    """
     A map of responses that correspond to individual upsert operations for custom attributes.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.terminal_checkout import TerminalCheckoutParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_terminal_checkout_response import CreateTerminalCheckoutResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.terminal_checkout_query import TerminalCheckoutQueryParams
from ...types.search_terminal_checkouts_response import SearchTerminalCheckoutsResponse
from ...types.get_terminal_checkout_response import GetTerminalCheckoutResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...types.cancel_terminal_checkout_response import CancelTerminalCheckoutResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCheckoutsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        idempotency_key: str,
        checkout: TerminalCheckoutParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateTerminalCheckoutResponse]:
        """
        Creates a Terminal checkout request and sends it to the specified device to take a payment
        for the requested amount.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateCheckout` request. Keys can be any valid string but
            must be unique for every `CreateCheckout` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        checkout : TerminalCheckoutParams
            The checkout to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateTerminalCheckoutResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/terminals/checkouts",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "checkout": convert_and_respect_annotation_metadata(
                    object_=checkout, annotation=TerminalCheckoutParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateTerminalCheckoutResponse,
                    construct_type(
                        type_=CreateTerminalCheckoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        query: typing.Optional[TerminalCheckoutQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchTerminalCheckoutsResponse]:
        """
        Returns a filtered list of Terminal checkout requests created by the application making the request. Only Terminal checkout requests created for the merchant scoped to the OAuth token are returned. Terminal checkout requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalCheckoutQueryParams]
            Queries Terminal checkouts based on given conditions and the sort order.
            Leaving these unset returns all checkouts with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        limit : typing.Optional[int]
            Limits the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchTerminalCheckoutsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/terminals/checkouts/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=TerminalCheckoutQueryParams, direction="write"
                ),
                "cursor": cursor,
                "limit": limit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchTerminalCheckoutsResponse,
                    construct_type(
                        type_=SearchTerminalCheckoutsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetTerminalCheckoutResponse]:
        """
        Retrieves a Terminal checkout request by `checkout_id`. Terminal checkout requests are available for 30 days.

        Parameters
        ----------
        checkout_id : str
            The unique ID for the desired `TerminalCheckout`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetTerminalCheckoutResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/terminals/checkouts/{jsonable_encoder(checkout_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTerminalCheckoutResponse,
                    construct_type(
                        type_=GetTerminalCheckoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CancelTerminalCheckoutResponse]:
        """
        Cancels a Terminal checkout request if the status of the request permits it.

        Parameters
        ----------
        checkout_id : str
            The unique ID for the desired `TerminalCheckout`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CancelTerminalCheckoutResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/terminals/checkouts/{jsonable_encoder(checkout_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelTerminalCheckoutResponse,
                    construct_type(
                        type_=CancelTerminalCheckoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCheckoutsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        idempotency_key: str,
        checkout: TerminalCheckoutParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateTerminalCheckoutResponse]:
        """
        Creates a Terminal checkout request and sends it to the specified device to take a payment
        for the requested amount.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateCheckout` request. Keys can be any valid string but
            must be unique for every `CreateCheckout` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        checkout : TerminalCheckoutParams
            The checkout to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateTerminalCheckoutResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/terminals/checkouts",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "checkout": convert_and_respect_annotation_metadata(
                    object_=checkout, annotation=TerminalCheckoutParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateTerminalCheckoutResponse,
                    construct_type(
                        type_=CreateTerminalCheckoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        query: typing.Optional[TerminalCheckoutQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchTerminalCheckoutsResponse]:
        """
        Returns a filtered list of Terminal checkout requests created by the application making the request. Only Terminal checkout requests created for the merchant scoped to the OAuth token are returned. Terminal checkout requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalCheckoutQueryParams]
            Queries Terminal checkouts based on given conditions and the sort order.
            Leaving these unset returns all checkouts with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        limit : typing.Optional[int]
            Limits the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchTerminalCheckoutsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/terminals/checkouts/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=TerminalCheckoutQueryParams, direction="write"
                ),
                "cursor": cursor,
                "limit": limit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchTerminalCheckoutsResponse,
                    construct_type(
                        type_=SearchTerminalCheckoutsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetTerminalCheckoutResponse]:
        """
        Retrieves a Terminal checkout request by `checkout_id`. Terminal checkout requests are available for 30 days.

        Parameters
        ----------
        checkout_id : str
            The unique ID for the desired `TerminalCheckout`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetTerminalCheckoutResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/terminals/checkouts/{jsonable_encoder(checkout_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTerminalCheckoutResponse,
                    construct_type(
                        type_=GetTerminalCheckoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CancelTerminalCheckoutResponse]:
        """
        Cancels a Terminal checkout request if the status of the request permits it.

        Parameters
        ----------
        checkout_id : str
            The unique ID for the desired `TerminalCheckout`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CancelTerminalCheckoutResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/terminals/checkouts/{jsonable_encoder(checkout_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelTerminalCheckoutResponse,
                    construct_type(
                        type_=CancelTerminalCheckoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.business_booking_profile_customer_timezone_choice import BusinessBookingProfileCustomerTimezoneChoice
from ..types.business_booking_profile_booking_policy import BusinessBookingProfileBookingPolicy
from .business_appointment_settings import BusinessAppointmentSettingsParams


class BusinessBookingProfileParams(typing_extensions.TypedDict):
    """
    A seller's business booking profile, including booking policy, appointment settings, etc.
    """

    seller_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the seller, obtainable using the Merchants API.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The RFC 3339 timestamp specifying the booking's creation time.
    """

    booking_enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the seller is open for booking.
    """

    customer_timezone_choice: typing_extensions.NotRequired[BusinessBookingProfileCustomerTimezoneChoice]
    """
    The choice of customer's time zone information of a booking.
    The Square online booking site and all notifications to customers uses either the seller location’s time zone
    or the time zone the customer chooses at booking.
    See [BusinessBookingProfileCustomerTimezoneChoice](#type-businessbookingprofilecustomertimezonechoice) for possible values
    """

    booking_policy: typing_extensions.NotRequired[BusinessBookingProfileBookingPolicy]
    """
    The policy for the seller to automatically accept booking requests (`ACCEPT_ALL`) or not (`REQUIRES_ACCEPTANCE`).
    See [BusinessBookingProfileBookingPolicy](#type-businessbookingprofilebookingpolicy) for possible values
    """

    allow_user_cancel: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether customers can cancel or reschedule their own bookings (`true`) or not (`false`).
    """

    business_appointment_settings: typing_extensions.NotRequired[BusinessAppointmentSettingsParams]
    """
    Settings for appointment-type bookings.
    """

    support_seller_level_writes: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the seller's subscription to Square Appointments supports creating, updating or canceling an appointment through the API (`true`) or not (`false`) using seller permission.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .team_member import TeamMemberParams


class UpdateTeamMemberRequestParams(typing_extensions.TypedDict):
    """
    Represents an update request for a `TeamMember` object.
    """

    team_member: typing_extensions.NotRequired[TeamMemberParams]
    """
    The team member fields to add, change, or clear. Fields can be cleared using a null value. To update
    `wage_setting.job_assignments`, you must provide the complete list of job assignments. If needed, call
    [ListJobs](api-endpoint:Team-ListJobs) to get the required `job_id` values.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .checkout_location_settings import CheckoutLocationSettingsParams


class RetrieveLocationSettingsResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    location_settings: typing_extensions.NotRequired[CheckoutLocationSettingsParams]
    """
    The location settings.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

InventoryState = typing.Union[
    typing.Literal[
        "CUSTOM",
        "IN_STOCK",
        "SOLD",
        "RETURNED_BY_CUSTOMER",
        "RESERVED_FOR_SALE",
        "SOLD_ONLINE",
        "ORDERED_FROM_VENDOR",
        "RECEIVED_FROM_VENDOR",
        "IN_TRANSIT_TO",
        "NONE",
        "WASTE",
        "UNLINKED_RETURN",
        "COMPOSED",
        "DECOMPOSED",
        "SUPPORTED_BY_NEWER_VERSION",
        "IN_TRANSIT",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .customer_filter import CustomerFilter
import pydantic
from .customer_sort import CustomerSort
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerQuery(UncheckedBaseModel):
    """
    Represents filtering and sorting criteria for a [SearchCustomers](api-endpoint:Customers-SearchCustomers) request.
    """

    filter: typing.Optional[CustomerFilter] = pydantic.Field(default=None)
    """
    The filtering criteria for the search query. A query can contain multiple filters in any combination.
    Multiple filters are combined as `AND` statements.
    
    __Note:__ Combining multiple filters as `OR` statements is not supported. Instead, send multiple single-filter
    searches and join the result sets.
    """

    sort: typing.Optional[CustomerSort] = pydantic.Field(default=None)
    """
    Sorting criteria for query results. The default behavior is to sort 
    customers alphabetically by `given_name` and `family_name`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogObjectType = typing.Union[
    typing.Literal[
        "ITEM",
        "IMAGE",
        "CATEGORY",
        "ITEM_VARIATION",
        "TAX",
        "DISCOUNT",
        "MODIFIER_LIST",
        "MODIFIER",
        "PRICING_RULE",
        "PRODUCT_SET",
        "TIME_PERIOD",
        "MEASUREMENT_UNIT",
        "SUBSCRIPTION_PLAN_VARIATION",
        "ITEM_OPTION",
        "ITEM_OPTION_VAL",
        "CUSTOM_ATTRIBUTE_DEFINITION",
        "QUICK_AMOUNTS_SETTINGS",
        "SUBSCRIPTION_PLAN",
        "AVAILABILITY_PERIOD",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.device_status_category import DeviceStatusCategory


class DeviceStatusParams(typing_extensions.TypedDict):
    category: typing_extensions.NotRequired[DeviceStatusCategory]
    """
    
    See [Category](#type-category) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ShiftWage(UncheckedBaseModel):
    """
    The hourly wage rate used to compensate an employee for this shift.

    Deprecated at Square API version 2025-05-21. See the [migration notes](https://developer.squareup.com/docs/labor-api/what-it-does#migration-notes).
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the job performed during this shift.
    """

    hourly_rate: typing.Optional[Money] = pydantic.Field(default=None)
    """
    Can be a custom-set hourly wage or the calculated effective hourly
    wage based on the annual wage and hours worked per week.
    """

    job_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the job performed during this shift. Square
    labor-reporting UIs might group shifts together by id.
    """

    tip_eligible: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether team members are eligible for tips when working this job.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams
import typing_extensions
import typing
from ..types.loyalty_program_accrual_rule_tax_mode import LoyaltyProgramAccrualRuleTaxMode


class LoyaltyProgramAccrualRuleSpendDataParams(typing_extensions.TypedDict):
    """
    Represents additional data for rules with the `SPEND` accrual type.
    """

    amount_money: MoneyParams
    """
    The amount that buyers must spend to earn points. 
    For example, given an "Earn 1 point for every $10 spent" accrual rule, a buyer who spends $105 earns 10 points.
    """

    excluded_category_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The IDs of any `CATEGORY` catalog objects that are excluded from points accrual.
    
    You can use the [BatchRetrieveCatalogObjects](api-endpoint:Catalog-BatchRetrieveCatalogObjects)
    endpoint to retrieve information about the excluded categories.
    """

    excluded_item_variation_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The IDs of any `ITEM_VARIATION` catalog objects that are excluded from points accrual.
    
    You can use the [BatchRetrieveCatalogObjects](api-endpoint:Catalog-BatchRetrieveCatalogObjects)
    endpoint to retrieve information about the excluded item variations.
    """

    tax_mode: LoyaltyProgramAccrualRuleTaxMode
    """
    Indicates how taxes should be treated when calculating the purchase amount used for points accrual.
    See [LoyaltyProgramAccrualRuleTaxMode](#type-loyaltyprogramaccrualruletaxmode) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing

BusinessAppointmentSettingsMaxAppointmentsPerDayLimitType = typing.Union[
    typing.Literal["PER_TEAM_MEMBER", "PER_LOCATION"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkDeleteOrderCustomAttributesRequestDeleteCustomAttribute(UncheckedBaseModel):
    """
    Represents one delete within the bulk operation.
    """

    key: typing.Optional[str] = pydantic.Field(default=None)
    """
    The key of the custom attribute to delete.  This key must match the key 
    of an existing custom attribute definition.
    """

    order_id: str = pydantic.Field()
    """
    The ID of the target [order](entity:Order).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .webhook_subscription import WebhookSubscription
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GetWebhookSubscriptionResponse(UncheckedBaseModel):
    """
    Defines the fields that are included in the response body of
    a request to the [RetrieveWebhookSubscription](api-endpoint:WebhookSubscriptions-RetrieveWebhookSubscription) endpoint.

    Note: if there are errors processing the request, the [Subscription](entity:WebhookSubscription) will not be
    present.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Information on errors encountered during the request.
    """

    subscription: typing.Optional[WebhookSubscription] = pydantic.Field(default=None)
    """
    The requested [Subscription](entity:WebhookSubscription).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class SearchOrdersSourceFilterParams(typing_extensions.TypedDict):
    """
    A filter based on order `source` information.
    """

    source_names: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Filters by the [Source](entity:OrderSource) `name`. The filter returns any orders
    with a `source.name` that matches any of the listed source names.
    
    Max: 10 source names.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.booking_status import BookingStatus
import typing
from .appointment_segment import AppointmentSegmentParams
from ..types.business_appointment_settings_booking_location_type import BusinessAppointmentSettingsBookingLocationType
from .booking_creator_details import BookingCreatorDetailsParams
from ..types.booking_booking_source import BookingBookingSource
from .address import AddressParams


class BookingParams(typing_extensions.TypedDict):
    """
    Represents a booking as a time-bound service contract for a seller's staff member to provide a specified service
    at a given location to a requesting customer in one or more appointment segments.
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique ID of this object representing a booking.
    """

    version: typing_extensions.NotRequired[int]
    """
    The revision number for the booking used for optimistic concurrency.
    """

    status: typing_extensions.NotRequired[BookingStatus]
    """
    The status of the booking, describing where the booking stands with respect to the booking state machine.
    See [BookingStatus](#type-bookingstatus) for possible values
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The RFC 3339 timestamp specifying the creation time of this booking.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The RFC 3339 timestamp specifying the most recent update time of this booking.
    """

    start_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The RFC 3339 timestamp specifying the starting time of this booking.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [Location](entity:Location) object representing the location where the booked service is provided. Once set when the booking is created, its value cannot be changed.
    """

    customer_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [Customer](entity:Customer) object representing the customer receiving the booked service.
    """

    customer_note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The free-text field for the customer to supply notes about the booking. For example, the note can be preferences that cannot be expressed by supported attributes of a relevant [CatalogObject](entity:CatalogObject) instance.
    """

    seller_note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The free-text field for the seller to supply notes about the booking. For example, the note can be preferences that cannot be expressed by supported attributes of a specific [CatalogObject](entity:CatalogObject) instance.
    This field should not be visible to customers.
    """

    appointment_segments: typing_extensions.NotRequired[typing.Optional[typing.Sequence[AppointmentSegmentParams]]]
    """
    A list of appointment segments for this booking.
    """

    transition_time_minutes: typing_extensions.NotRequired[int]
    """
    Additional time at the end of a booking.
    Applications should not make this field visible to customers of a seller.
    """

    all_day: typing_extensions.NotRequired[bool]
    """
    Whether the booking is of a full business day.
    """

    location_type: typing_extensions.NotRequired[BusinessAppointmentSettingsBookingLocationType]
    """
    The type of location where the booking is held.
    See [BusinessAppointmentSettingsBookingLocationType](#type-businessappointmentsettingsbookinglocationtype) for possible values
    """

    creator_details: typing_extensions.NotRequired[BookingCreatorDetailsParams]
    """
    Information about the booking creator.
    """

    source: typing_extensions.NotRequired[BookingBookingSource]
    """
    The source of the booking.
    Access to this field requires seller-level permissions.
    See [BookingBookingSource](#type-bookingbookingsource) for possible values
    """

    address: typing_extensions.NotRequired[AddressParams]
    """
    Stores a customer address if the location type is `CUSTOMER_LOCATION`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .order import OrderParams
from .catalog_object import CatalogObjectParams


class PaymentLinkRelatedResourcesParams(typing_extensions.TypedDict):
    orders: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderParams]]]
    """
    The order associated with the payment link.
    """

    subscription_plans: typing_extensions.NotRequired[typing.Optional[typing.Sequence[CatalogObjectParams]]]
    """
    The subscription plan associated with the payment link.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .scheduled_shift_details import ScheduledShiftDetails
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ScheduledShift(UncheckedBaseModel):
    """
    Represents a specific time slot in a work schedule. This object is used to manage the
    lifecycle of a scheduled shift from the draft to published state. A scheduled shift contains
    the latest draft shift details and current published shift details.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    **Read only** The Square-issued ID of the scheduled shift.
    """

    draft_shift_details: typing.Optional[ScheduledShiftDetails] = pydantic.Field(default=None)
    """
    The latest draft shift details for the scheduled shift. Draft shift details are used to
    stage and manage shifts before publishing. This field is always present.
    """

    published_shift_details: typing.Optional[ScheduledShiftDetails] = pydantic.Field(default=None)
    """
    The current published (public) shift details for the scheduled shift. This field is
    present only if the shift was published.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    **Read only** The current version of the scheduled shift, which is incremented with each update.
    This field is used for [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
    control to ensure that requests don't overwrite data from another request.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the scheduled shift was created, in RFC 3339 format presented as UTC.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the scheduled shift was last updated, in RFC 3339 format presented as UTC.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing
from ..types.order_state import OrderState


class SearchOrdersStateFilterParams(typing_extensions.TypedDict):
    """
    Filter by the current order `state`.
    """

    states: typing.Sequence[OrderState]
    """
    States to filter for.
    See [OrderState](#type-orderstate) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogCustomAttributeDefinitionNumberConfigParams(typing_extensions.TypedDict):
    precision: typing_extensions.NotRequired[typing.Optional[int]]
    """
    An integer between 0 and 5 that represents the maximum number of
    positions allowed after the decimal in number custom attribute values
    For example:
    
    - if the precision is 0, the quantity can be 1, 2, 3, etc.
    - if the precision is 1, the quantity can be 0.1, 0.2, etc.
    - if the precision is 2, the quantity can be 0.01, 0.12, etc.
    
    Default: 5
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.order_line_item_tax_type import OrderLineItemTaxType
from .money import MoneyParams
from ..types.order_line_item_tax_scope import OrderLineItemTaxScope


class OrderLineItemTaxParams(typing_extensions.TypedDict):
    """
    Represents a tax that applies to one or more line item in the order.

    Fixed-amount, order-scoped taxes are distributed across all non-zero line item totals.
    The amount distributed to each line item is relative to the amount the item
    contributes to the order subtotal.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the tax only within this order.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The catalog object ID referencing [CatalogTax](entity:CatalogTax).
    """

    catalog_version: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The version of the catalog object that this tax references.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The tax's name.
    """

    type: typing_extensions.NotRequired[OrderLineItemTaxType]
    """
    Indicates the calculation method used to apply the tax.
    See [OrderLineItemTaxType](#type-orderlineitemtaxtype) for possible values
    """

    percentage: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The percentage of the tax, as a string representation of a decimal
    number. For example, a value of `"7.25"` corresponds to a percentage of
    7.25%.
    """

    metadata: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[str]]]]
    """
    Application-defined data attached to this tax. Metadata fields are intended
    to store descriptive references or associations with an entity in another system or store brief
    information about the object. Square does not process this field; it only stores and returns it
    in relevant API calls. Do not use metadata to store any sensitive information (such as personally
    identifiable information or card details).
    
    Keys written by applications must be 60 characters or less and must be in the character set
    `[a-zA-Z0-9_-]`. Entries can also include metadata generated by Square. These keys are prefixed
    with a namespace, separated from the key with a ':' character.
    
    Values have a maximum length of 255 characters.
    
    An application can have up to 10 entries per metadata field.
    
    Entries written by applications are private and can only be read or modified by the same
    application.
    
    For more information, see [Metadata](https://developer.squareup.com/docs/build-basics/metadata).
    """

    applied_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money applied to the order by the tax.
    
    - For percentage-based taxes, `applied_money` is the money
    calculated using the percentage.
    """

    scope: typing_extensions.NotRequired[OrderLineItemTaxScope]
    """
    Indicates the level at which the tax applies. For `ORDER` scoped taxes,
    Square generates references in `applied_taxes` on all order line items that do
    not have them. For `LINE_ITEM` scoped taxes, the tax only applies to line items
    with references in their `applied_taxes` field.
    
    This field is immutable. To change the scope, you must delete the tax and
    re-add it as a new tax.
    See [OrderLineItemTaxScope](#type-orderlineitemtaxscope) for possible values
    """

    auto_applied: typing_extensions.NotRequired[bool]
    """
    Determines whether the tax was automatically applied to the order based on
    the catalog configuration. For an example, see
    [Automatically Apply Taxes to an Order](https://developer.squareup.com/docs/orders-api/apply-taxes-and-discounts/auto-apply-taxes).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .signature_image import SignatureImage
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SignatureOptions(UncheckedBaseModel):
    title: str = pydantic.Field()
    """
    The title text to display in the signature capture flow on the Terminal.
    """

    body: str = pydantic.Field()
    """
    The body text to display in the signature capture flow on the Terminal.
    """

    signature: typing.Optional[typing.List[SignatureImage]] = pydantic.Field(default=None)
    """
    An image representation of the collected signature.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class ConfirmationDecisionParams(typing_extensions.TypedDict):
    has_agreed: typing_extensions.NotRequired[bool]
    """
    The buyer's decision to the displayed terms.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .job import JobParams
import typing
from .error import ErrorParams


class RetrieveJobResponseParams(typing_extensions.TypedDict):
    """
    Represents a [RetrieveJob](api-endpoint:Team-RetrieveJob) response. Either `job` or `errors`
    is present in the response.
    """

    job: typing_extensions.NotRequired[JobParams]
    """
    The retrieved job.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    The errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

ApplicationDetailsExternalSquareProduct = typing.Union[
    typing.Literal[
        "APPOINTMENTS",
        "ECOMMERCE_API",
        "INVOICES",
        "ONLINE_STORE",
        "OTHER",
        "RESTAURANTS",
        "RETAIL",
        "SQUARE_POS",
        "TERMINAL_API",
        "VIRTUAL_TERMINAL",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .update_vendor_response import UpdateVendorResponseParams


class BatchUpdateVendorsResponseParams(typing_extensions.TypedDict):
    """
    Represents an output from a call to [BulkUpdateVendors](api-endpoint:Vendors-BulkUpdateVendors).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors encountered when the request fails.
    """

    responses: typing_extensions.NotRequired[typing.Dict[str, UpdateVendorResponseParams]]
    """
    A set of [UpdateVendorResponse](entity:UpdateVendorResponse) objects encapsulating successfully created [Vendor](entity:Vendor)
    objects or error responses for failed attempts. The set is represented by a collection of `Vendor`-ID/`UpdateVendorResponse`-object or 
    `Vendor`-ID/error-object pairs.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .business_booking_profile_customer_timezone_choice import BusinessBookingProfileCustomerTimezoneChoice
from .business_booking_profile_booking_policy import BusinessBookingProfileBookingPolicy
from .business_appointment_settings import BusinessAppointmentSettings
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BusinessBookingProfile(UncheckedBaseModel):
    """
    A seller's business booking profile, including booking policy, appointment settings, etc.
    """

    seller_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the seller, obtainable using the Merchants API.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The RFC 3339 timestamp specifying the booking's creation time.
    """

    booking_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the seller is open for booking.
    """

    customer_timezone_choice: typing.Optional[BusinessBookingProfileCustomerTimezoneChoice] = pydantic.Field(
        default=None
    )
    """
    The choice of customer's time zone information of a booking.
    The Square online booking site and all notifications to customers uses either the seller location’s time zone
    or the time zone the customer chooses at booking.
    See [BusinessBookingProfileCustomerTimezoneChoice](#type-businessbookingprofilecustomertimezonechoice) for possible values
    """

    booking_policy: typing.Optional[BusinessBookingProfileBookingPolicy] = pydantic.Field(default=None)
    """
    The policy for the seller to automatically accept booking requests (`ACCEPT_ALL`) or not (`REQUIRES_ACCEPTANCE`).
    See [BusinessBookingProfileBookingPolicy](#type-businessbookingprofilebookingpolicy) for possible values
    """

    allow_user_cancel: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether customers can cancel or reschedule their own bookings (`true`) or not (`false`).
    """

    business_appointment_settings: typing.Optional[BusinessAppointmentSettings] = pydantic.Field(default=None)
    """
    Settings for appointment-type bookings.
    """

    support_seller_level_writes: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the seller's subscription to Square Appointments supports creating, updating or canceling an appointment through the API (`true`) or not (`false`) using seller permission.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawBookingsClient
from .custom_attribute_definitions.client import CustomAttributeDefinitionsClient
from .custom_attributes.client import CustomAttributesClient
from .location_profiles.client import LocationProfilesClient
from .team_member_profiles.client import TeamMemberProfilesClient
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.booking import Booking
from ..types.list_bookings_response import ListBookingsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.booking import BookingParams
from ..types.create_booking_response import CreateBookingResponse
from ..requests.search_availability_query import SearchAvailabilityQueryParams
from ..types.search_availability_response import SearchAvailabilityResponse
from ..types.bulk_retrieve_bookings_response import BulkRetrieveBookingsResponse
from ..types.get_business_booking_profile_response import GetBusinessBookingProfileResponse
from ..types.retrieve_location_booking_profile_response import RetrieveLocationBookingProfileResponse
from ..types.bulk_retrieve_team_member_booking_profiles_response import BulkRetrieveTeamMemberBookingProfilesResponse
from ..types.get_booking_response import GetBookingResponse
from ..types.update_booking_response import UpdateBookingResponse
from ..types.cancel_booking_response import CancelBookingResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawBookingsClient
from .custom_attribute_definitions.client import AsyncCustomAttributeDefinitionsClient
from .custom_attributes.client import AsyncCustomAttributesClient
from .location_profiles.client import AsyncLocationProfilesClient
from .team_member_profiles.client import AsyncTeamMemberProfilesClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BookingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBookingsClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = CustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.custom_attributes = CustomAttributesClient(client_wrapper=client_wrapper)

        self.location_profiles = LocationProfilesClient(client_wrapper=client_wrapper)

        self.team_member_profiles = TeamMemberProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBookingsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBookingsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        customer_id: typing.Optional[str] = None,
        team_member_id: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        start_at_min: typing.Optional[str] = None,
        start_at_max: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Booking]:
        """
        Retrieve a collection of bookings.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        limit : typing.Optional[int]
            The maximum number of results per page to return in a paged response.

        cursor : typing.Optional[str]
            The pagination cursor from the preceding response to return the next page of the results. Do not set this when retrieving the first page of the results.

        customer_id : typing.Optional[str]
            The [customer](entity:Customer) for whom to retrieve bookings. If this is not set, bookings for all customers are retrieved.

        team_member_id : typing.Optional[str]
            The team member for whom to retrieve bookings. If this is not set, bookings of all members are retrieved.

        location_id : typing.Optional[str]
            The location for which to retrieve bookings. If this is not set, all locations' bookings are retrieved.

        start_at_min : typing.Optional[str]
            The RFC 3339 timestamp specifying the earliest of the start time. If this is not set, the current time is used.

        start_at_max : typing.Optional[str]
            The RFC 3339 timestamp specifying the latest of the start time. If this is not set, the time of 31 days after `start_at_min` is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Booking]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.bookings.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/bookings",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
                "customer_id": customer_id,
                "team_member_id": team_member_id,
                "location_id": location_id,
                "start_at_min": start_at_min,
                "start_at_max": start_at_max,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBookingsResponse,
                    construct_type(
                        type_=ListBookingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    limit=limit,
                    cursor=_parsed_next,
                    customer_id=customer_id,
                    team_member_id=team_member_id,
                    location_id=location_id,
                    start_at_min=start_at_min,
                    start_at_max=start_at_max,
                    request_options=request_options,
                )
                _items = _parsed_response.bookings
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        booking: BookingParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBookingResponse:
        """
        Creates a booking.

        The required input must include the following:
        - `Booking.location_id`
        - `Booking.start_at`
        - `Booking.AppointmentSegment.team_member_id`
        - `Booking.AppointmentSegment.service_variation_id`
        - `Booking.AppointmentSegment.service_variation_version`

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking : BookingParams
            The details of the booking to be created.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBookingResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.create(
            booking={},
        )
        """
        response = self._raw_client.create(
            booking=booking, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def search_availability(
        self, *, query: SearchAvailabilityQueryParams, request_options: typing.Optional[RequestOptions] = None
    ) -> SearchAvailabilityResponse:
        """
        Searches for availabilities for booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        query : SearchAvailabilityQueryParams
            Query conditions used to filter buyer-accessible booking availabilities.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchAvailabilityResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.search_availability(
            query={"filter": {"start_at_range": {}}},
        )
        """
        response = self._raw_client.search_availability(query=query, request_options=request_options)
        return response.data

    def bulk_retrieve_bookings(
        self, *, booking_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> BulkRetrieveBookingsResponse:
        """
        Bulk-Retrieves a list of bookings by booking IDs.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_ids : typing.Sequence[str]
            A non-empty list of [Booking](entity:Booking) IDs specifying bookings to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkRetrieveBookingsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.bulk_retrieve_bookings(
            booking_ids=["booking_ids"],
        )
        """
        response = self._raw_client.bulk_retrieve_bookings(booking_ids=booking_ids, request_options=request_options)
        return response.data

    def get_business_profile(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBusinessBookingProfileResponse:
        """
        Retrieves a seller's booking profile.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBusinessBookingProfileResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.get_business_profile()
        """
        response = self._raw_client.get_business_profile(request_options=request_options)
        return response.data

    def retrieve_location_booking_profile(
        self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveLocationBookingProfileResponse:
        """
        Retrieves a seller's location booking profile.

        Parameters
        ----------
        location_id : str
            The ID of the location to retrieve the booking profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveLocationBookingProfileResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.retrieve_location_booking_profile(
            location_id="location_id",
        )
        """
        response = self._raw_client.retrieve_location_booking_profile(location_id, request_options=request_options)
        return response.data

    def bulk_retrieve_team_member_booking_profiles(
        self, *, team_member_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> BulkRetrieveTeamMemberBookingProfilesResponse:
        """
        Retrieves one or more team members' booking profiles.

        Parameters
        ----------
        team_member_ids : typing.Sequence[str]
            A non-empty list of IDs of team members whose booking profiles you want to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkRetrieveTeamMemberBookingProfilesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.bulk_retrieve_team_member_booking_profiles(
            team_member_ids=["team_member_ids"],
        )
        """
        response = self._raw_client.bulk_retrieve_team_member_booking_profiles(
            team_member_ids=team_member_ids, request_options=request_options
        )
        return response.data

    def get(self, booking_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetBookingResponse:
        """
        Retrieves a booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-retrieved booking.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBookingResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.get(
            booking_id="booking_id",
        )
        """
        response = self._raw_client.get(booking_id, request_options=request_options)
        return response.data

    def update(
        self,
        booking_id: str,
        *,
        booking: BookingParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBookingResponse:
        """
        Updates a booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-updated booking.

        booking : BookingParams
            The booking to be updated. Individual attributes explicitly specified here override the corresponding values of the existing booking.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBookingResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.update(
            booking_id="booking_id",
            booking={},
        )
        """
        response = self._raw_client.update(
            booking_id, booking=booking, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def cancel(
        self,
        booking_id: str,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        booking_version: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CancelBookingResponse:
        """
        Cancels an existing booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-cancelled booking.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        booking_version : typing.Optional[int]
            The revision number for the booking used for optimistic concurrency.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelBookingResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.cancel(
            booking_id="booking_id",
        )
        """
        response = self._raw_client.cancel(
            booking_id,
            idempotency_key=idempotency_key,
            booking_version=booking_version,
            request_options=request_options,
        )
        return response.data


class AsyncBookingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBookingsClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = AsyncCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.custom_attributes = AsyncCustomAttributesClient(client_wrapper=client_wrapper)

        self.location_profiles = AsyncLocationProfilesClient(client_wrapper=client_wrapper)

        self.team_member_profiles = AsyncTeamMemberProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBookingsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBookingsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        customer_id: typing.Optional[str] = None,
        team_member_id: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        start_at_min: typing.Optional[str] = None,
        start_at_max: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Booking]:
        """
        Retrieve a collection of bookings.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        limit : typing.Optional[int]
            The maximum number of results per page to return in a paged response.

        cursor : typing.Optional[str]
            The pagination cursor from the preceding response to return the next page of the results. Do not set this when retrieving the first page of the results.

        customer_id : typing.Optional[str]
            The [customer](entity:Customer) for whom to retrieve bookings. If this is not set, bookings for all customers are retrieved.

        team_member_id : typing.Optional[str]
            The team member for whom to retrieve bookings. If this is not set, bookings of all members are retrieved.

        location_id : typing.Optional[str]
            The location for which to retrieve bookings. If this is not set, all locations' bookings are retrieved.

        start_at_min : typing.Optional[str]
            The RFC 3339 timestamp specifying the earliest of the start time. If this is not set, the current time is used.

        start_at_max : typing.Optional[str]
            The RFC 3339 timestamp specifying the latest of the start time. If this is not set, the time of 31 days after `start_at_min` is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Booking]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.bookings.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/bookings",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
                "customer_id": customer_id,
                "team_member_id": team_member_id,
                "location_id": location_id,
                "start_at_min": start_at_min,
                "start_at_max": start_at_max,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBookingsResponse,
                    construct_type(
                        type_=ListBookingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    limit=limit,
                    cursor=_parsed_next,
                    customer_id=customer_id,
                    team_member_id=team_member_id,
                    location_id=location_id,
                    start_at_min=start_at_min,
                    start_at_max=start_at_max,
                    request_options=request_options,
                )
                _items = _parsed_response.bookings
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        booking: BookingParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBookingResponse:
        """
        Creates a booking.

        The required input must include the following:
        - `Booking.location_id`
        - `Booking.start_at`
        - `Booking.AppointmentSegment.team_member_id`
        - `Booking.AppointmentSegment.service_variation_id`
        - `Booking.AppointmentSegment.service_variation_version`

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking : BookingParams
            The details of the booking to be created.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBookingResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.create(
                booking={},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            booking=booking, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def search_availability(
        self, *, query: SearchAvailabilityQueryParams, request_options: typing.Optional[RequestOptions] = None
    ) -> SearchAvailabilityResponse:
        """
        Searches for availabilities for booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        query : SearchAvailabilityQueryParams
            Query conditions used to filter buyer-accessible booking availabilities.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchAvailabilityResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.search_availability(
                query={"filter": {"start_at_range": {}}},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search_availability(query=query, request_options=request_options)
        return response.data

    async def bulk_retrieve_bookings(
        self, *, booking_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> BulkRetrieveBookingsResponse:
        """
        Bulk-Retrieves a list of bookings by booking IDs.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_ids : typing.Sequence[str]
            A non-empty list of [Booking](entity:Booking) IDs specifying bookings to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkRetrieveBookingsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.bulk_retrieve_bookings(
                booking_ids=["booking_ids"],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.bulk_retrieve_bookings(
            booking_ids=booking_ids, request_options=request_options
        )
        return response.data

    async def get_business_profile(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBusinessBookingProfileResponse:
        """
        Retrieves a seller's booking profile.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBusinessBookingProfileResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.get_business_profile()


        asyncio.run(main())
        """
        response = await self._raw_client.get_business_profile(request_options=request_options)
        return response.data

    async def retrieve_location_booking_profile(
        self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveLocationBookingProfileResponse:
        """
        Retrieves a seller's location booking profile.

        Parameters
        ----------
        location_id : str
            The ID of the location to retrieve the booking profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveLocationBookingProfileResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.retrieve_location_booking_profile(
                location_id="location_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.retrieve_location_booking_profile(
            location_id, request_options=request_options
        )
        return response.data

    async def bulk_retrieve_team_member_booking_profiles(
        self, *, team_member_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> BulkRetrieveTeamMemberBookingProfilesResponse:
        """
        Retrieves one or more team members' booking profiles.

        Parameters
        ----------
        team_member_ids : typing.Sequence[str]
            A non-empty list of IDs of team members whose booking profiles you want to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkRetrieveTeamMemberBookingProfilesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.bulk_retrieve_team_member_booking_profiles(
                team_member_ids=["team_member_ids"],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.bulk_retrieve_team_member_booking_profiles(
            team_member_ids=team_member_ids, request_options=request_options
        )
        return response.data

    async def get(
        self, booking_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBookingResponse:
        """
        Retrieves a booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-retrieved booking.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBookingResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.get(
                booking_id="booking_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(booking_id, request_options=request_options)
        return response.data

    async def update(
        self,
        booking_id: str,
        *,
        booking: BookingParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBookingResponse:
        """
        Updates a booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-updated booking.

        booking : BookingParams
            The booking to be updated. Individual attributes explicitly specified here override the corresponding values of the existing booking.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBookingResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.update(
                booking_id="booking_id",
                booking={},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            booking_id, booking=booking, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def cancel(
        self,
        booking_id: str,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        booking_version: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CancelBookingResponse:
        """
        Cancels an existing booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-cancelled booking.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        booking_version : typing.Optional[int]
            The revision number for the booking used for optimistic concurrency.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelBookingResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.cancel(
                booking_id="booking_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.cancel(
            booking_id,
            idempotency_key=idempotency_key,
            booking_version=booking_version,
            request_options=request_options,
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class SubscriptionTestResultParams(typing_extensions.TypedDict):
    """
    Represents the details of a webhook subscription, including notification URL,
    event types, and signature key.
    """

    id: typing_extensions.NotRequired[str]
    """
    A Square-generated unique ID for the subscription test result.
    """

    status_code: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The status code returned by the subscription notification URL.
    """

    payload: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An object containing the payload of the test event. For example, a `payment.created` event.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the subscription was created, in RFC 3339 format. 
    For example, "2016-09-04T23:59:33.123Z".
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the subscription was updated, in RFC 3339 format. For example, "2016-09-04T23:59:33.123Z".
    Because a subscription test result is unique, this field is the same as the `created_at` field.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class DeleteBookingCustomAttributeDefinitionResponseParams(typing_extensions.TypedDict):
    """
    Represents a [DeleteBookingCustomAttributeDefinition](api-endpoint:BookingCustomAttributes-DeleteBookingCustomAttributeDefinition) response
    containing error messages when errors occurred during the request. The successful response does not contain any payload.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .inventory_state import InventoryState
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InventoryAdjustmentGroup(UncheckedBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID generated by Square for the
    `InventoryAdjustmentGroup`.
    """

    root_adjustment_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The inventory adjustment of the composed variation.
    """

    from_state: typing.Optional[InventoryState] = pydantic.Field(default=None)
    """
    Representative `from_state` for adjustments within the group. For example, for a group adjustment from `IN_STOCK` to `SOLD`,
    there can be two component adjustments in the group: one from `IN_STOCK`to `COMPOSED` and the other one from `COMPOSED` to `SOLD`.
    Here, the representative `from_state` for the `InventoryAdjustmentGroup` is `IN_STOCK`.
    See [InventoryState](#type-inventorystate) for possible values
    """

    to_state: typing.Optional[InventoryState] = pydantic.Field(default=None)
    """
    Representative `to_state` for adjustments within group. For example, for a group adjustment from `IN_STOCK` to `SOLD`,
    the two component adjustments in the group can be from `IN_STOCK` to `COMPOSED` and from `COMPOSED` to `SOLD`.
    Here, the representative `to_state` of the `InventoryAdjustmentGroup` is `SOLD`.
    See [InventoryState](#type-inventorystate) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .weekday import Weekday
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class WorkweekConfig(UncheckedBaseModel):
    """
    Sets the day of the week and hour of the day that a business starts a
    workweek. This is used to calculate overtime pay.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The UUID for this object.
    """

    start_of_week: Weekday = pydantic.Field()
    """
    The day of the week on which a business week starts for
    compensation purposes.
    See [Weekday](#type-weekday) for possible values
    """

    start_of_day_local_time: str = pydantic.Field()
    """
    The local time at which a business week starts. Represented as a
    string in `HH:MM` format (`HH:MM:SS` is also accepted, but seconds are
    truncated).
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    Used for resolving concurrency issues. The request fails if the version
    provided does not match the server version at the time of the request. If not provided,
    Square executes a blind write; potentially overwriting data from another
    write.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A read-only timestamp in RFC 3339 format; presented in UTC.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A read-only timestamp in RFC 3339 format; presented in UTC.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class OrderLineItemPricingBlocklistsBlockedDiscountParams(typing_extensions.TypedDict):
    """
    A discount to block from applying to a line item. The discount must be
    identified by either `discount_uid` or `discount_catalog_object_id`, but not both.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID of the `BlockedDiscount` within the order.
    """

    discount_uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The `uid` of the discount that should be blocked. Use this field to block
    ad hoc discounts. For catalog discounts, use the `discount_catalog_object_id` field.
    """

    discount_catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The `catalog_object_id` of the discount that should be blocked.
    Use this field to block catalog discounts. For ad hoc discounts, use the
    `discount_uid` field.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .product import Product
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SourceApplication(UncheckedBaseModel):
    """
    Represents information about the application used to generate a change.
    """

    product: typing.Optional[Product] = pydantic.Field(default=None)
    """
    __Read only__ The [product](entity:Product) type of the application.
    See [Product](#type-product) for possible values
    """

    application_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    __Read only__ The Square-assigned ID of the application. This field is used only if the
    [product](entity:Product) type is `EXTERNAL_API`.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    __Read only__ The display name of the application
    (for example, `"Custom Application"` or `"Square POS 4.74 for Android"`).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class SearchOrdersCustomerFilterParams(typing_extensions.TypedDict):
    """
    A filter based on the order `customer_id` and any tender `customer_id`
    associated with the order. It does not filter based on the
    [FulfillmentRecipient](entity:FulfillmentRecipient) `customer_id`.
    """

    customer_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A list of customer IDs to filter by.
    
    Max: 10 customer ids.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .invoice import InvoiceParams
from .error import ErrorParams


class ListInvoicesResponseParams(typing_extensions.TypedDict):
    """
    Describes a `ListInvoice` response.
    """

    invoices: typing_extensions.NotRequired[typing.Sequence[InvoiceParams]]
    """
    The invoices retrieved.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    When a response is truncated, it includes a cursor that you can use in a 
    subsequent request to retrieve the next set of invoices. If empty, this is the final 
    response. 
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .confirmation_decision import ConfirmationDecisionParams


class ConfirmationOptionsParams(typing_extensions.TypedDict):
    title: str
    """
    The title text to display in the confirmation screen flow on the Terminal.
    """

    body: str
    """
    The agreement details to display in the confirmation flow on the Terminal.
    """

    agree_button_text: str
    """
    The button text to display indicating the customer agrees to the displayed terms.
    """

    disagree_button_text: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The button text to display indicating the customer does not agree to the displayed terms.
    """

    decision: typing_extensions.NotRequired[ConfirmationDecisionParams]
    """
    The result of the buyer’s actions when presented with the confirmation screen.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .search_events_filter import SearchEventsFilterParams
from .search_events_sort import SearchEventsSortParams


class SearchEventsQueryParams(typing_extensions.TypedDict):
    """
    Contains query criteria for the search.
    """

    filter: typing_extensions.NotRequired[SearchEventsFilterParams]
    """
    Criteria to filter events by.
    """

    sort: typing_extensions.NotRequired[SearchEventsSortParams]
    """
    Criteria to sort events by.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.loyalty_event_type import LoyaltyEventType
import typing_extensions
from .loyalty_event_accumulate_points import LoyaltyEventAccumulatePointsParams
from .loyalty_event_create_reward import LoyaltyEventCreateRewardParams
from .loyalty_event_redeem_reward import LoyaltyEventRedeemRewardParams
from .loyalty_event_delete_reward import LoyaltyEventDeleteRewardParams
from .loyalty_event_adjust_points import LoyaltyEventAdjustPointsParams
from ..types.loyalty_event_source import LoyaltyEventSource
from .loyalty_event_expire_points import LoyaltyEventExpirePointsParams
from .loyalty_event_other import LoyaltyEventOtherParams
from .loyalty_event_accumulate_promotion_points import LoyaltyEventAccumulatePromotionPointsParams


class LoyaltyEventParams(typing_extensions.TypedDict):
    """
    Provides information about a loyalty event.
    For more information, see [Search for Balance-Changing Loyalty Events](https://developer.squareup.com/docs/loyalty-api/loyalty-events).
    """

    id: str
    """
    The Square-assigned ID of the loyalty event.
    """

    type: LoyaltyEventType
    """
    The type of the loyalty event.
    See [LoyaltyEventType](#type-loyaltyeventtype) for possible values
    """

    created_at: str
    """
    The timestamp when the event was created, in RFC 3339 format.
    """

    accumulate_points: typing_extensions.NotRequired[LoyaltyEventAccumulatePointsParams]
    """
    Provides metadata when the event `type` is `ACCUMULATE_POINTS`.
    """

    create_reward: typing_extensions.NotRequired[LoyaltyEventCreateRewardParams]
    """
    Provides metadata when the event `type` is `CREATE_REWARD`.
    """

    redeem_reward: typing_extensions.NotRequired[LoyaltyEventRedeemRewardParams]
    """
    Provides metadata when the event `type` is `REDEEM_REWARD`.
    """

    delete_reward: typing_extensions.NotRequired[LoyaltyEventDeleteRewardParams]
    """
    Provides metadata when the event `type` is `DELETE_REWARD`.
    """

    adjust_points: typing_extensions.NotRequired[LoyaltyEventAdjustPointsParams]
    """
    Provides metadata when the event `type` is `ADJUST_POINTS`.
    """

    loyalty_account_id: str
    """
    The ID of the [loyalty account](entity:LoyaltyAccount) associated with the event.
    """

    location_id: typing_extensions.NotRequired[str]
    """
    The ID of the [location](entity:Location) where the event occurred.
    """

    source: LoyaltyEventSource
    """
    Defines whether the event was generated by the Square Point of Sale.
    See [LoyaltyEventSource](#type-loyaltyeventsource) for possible values
    """

    expire_points: typing_extensions.NotRequired[LoyaltyEventExpirePointsParams]
    """
    Provides metadata when the event `type` is `EXPIRE_POINTS`.
    """

    other_event: typing_extensions.NotRequired[LoyaltyEventOtherParams]
    """
    Provides metadata when the event `type` is `OTHER`.
    """

    accumulate_promotion_points: typing_extensions.NotRequired[LoyaltyEventAccumulatePromotionPointsParams]
    """
    Provides metadata when the event `type` is `ACCUMULATE_PROMOTION_POINTS`.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class FulfillmentPickupDetailsCurbsidePickupDetails(UncheckedBaseModel):
    """
    Specific details for curbside pickup.
    """

    curbside_details: typing.Optional[str] = pydantic.Field(default=None)
    """
    Specific details for curbside pickup, such as parking number and vehicle model.
    """

    buyer_arrived_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the buyer arrived and is waiting for pickup. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCustomAttributeDefinitionsClient
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.custom_attribute_definition import CustomAttributeDefinition
from ...types.list_customer_custom_attribute_definitions_response import ListCustomerCustomAttributeDefinitionsResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.custom_attribute_definition import CustomAttributeDefinitionParams
from ...types.create_customer_custom_attribute_definition_response import (
    CreateCustomerCustomAttributeDefinitionResponse,
)
from ...types.get_customer_custom_attribute_definition_response import GetCustomerCustomAttributeDefinitionResponse
from ...types.update_customer_custom_attribute_definition_response import (
    UpdateCustomerCustomAttributeDefinitionResponse,
)
from ...types.delete_customer_custom_attribute_definition_response import (
    DeleteCustomerCustomAttributeDefinitionResponse,
)
from ...requests.batch_upsert_customer_custom_attributes_request_customer_custom_attribute_upsert_request import (
    BatchUpsertCustomerCustomAttributesRequestCustomerCustomAttributeUpsertRequestParams,
)
from ...types.batch_upsert_customer_custom_attributes_response import BatchUpsertCustomerCustomAttributesResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCustomAttributeDefinitionsClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomAttributeDefinitionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomAttributeDefinitionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CustomAttributeDefinition]:
        """
        Lists the customer-related [custom attribute definitions](entity:CustomAttributeDefinition) that belong to a Square seller account.

        When all response pages are retrieved, the results include all custom attribute definitions
        that are visible to the requesting application, including those that are created by other
        applications and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that
        seller-defined custom attributes (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CustomAttributeDefinition]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.customers.custom_attribute_definitions.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/customers/custom-attribute-definitions",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCustomerCustomAttributeDefinitionsResponse,
                    construct_type(
                        type_=ListCustomerCustomAttributeDefinitionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attribute_definitions
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomerCustomAttributeDefinitionResponse:
        """
        Creates a customer-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to define a custom attribute that can be associated with customer profiles.

        A custom attribute definition specifies the `key`, `visibility`, `schema`, and other properties
        for a custom attribute. After the definition is created, you can call
        [UpsertCustomerCustomAttribute](api-endpoint:CustomerCustomAttributes-UpsertCustomerCustomAttribute) or
        [BulkUpsertCustomerCustomAttributes](api-endpoint:CustomerCustomAttributes-BulkUpsertCustomerCustomAttributes)
        to set the custom attribute for customer profiles in the seller's Customer Directory.

        Sellers can view all custom attributes in exported customer data, including those set to
        `VISIBILITY_HIDDEN`.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Specifying the schema](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attribute-definitions#specify-schema).
            - If provided, `name` must be unique (case-sensitive) across all visible customer-related custom attribute definitions for the seller.
            - All custom attributes are visible in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.custom_attribute_definitions.create(
            custom_attribute_definition={
                "key": "favoritemovie",
                "schema": {
                    "ref": "https://developer-production-s.squarecdn.com/schemas/v1/common.json#squareup.common.String"
                },
                "name": "Favorite Movie",
                "description": "The favorite movie of the customer.",
                "visibility": "VISIBILITY_HIDDEN",
            },
        )
        """
        response = self._raw_client.create(
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomerCustomAttributeDefinitionResponse:
        """
        Retrieves a customer-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomerCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.custom_attribute_definitions.get(
            key="key",
        )
        """
        response = self._raw_client.get(key, version=version, request_options=request_options)
        return response.data

    def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCustomerCustomAttributeDefinitionResponse:
        """
        Updates a customer-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.

        Use this endpoint to update the following fields: `name`, `description`, `visibility`, or the
        `schema` for a `Selection` data type.

        Only the definition owner can update a custom attribute definition. Note that sellers can view
        all custom attributes in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint
            supports sparse updates, so only new or changed fields need to be included in the request.
            Only the following fields can be updated:

            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCustomerCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.custom_attribute_definitions.update(
            key="key",
            custom_attribute_definition={
                "description": "Update the description as desired.",
                "visibility": "VISIBILITY_READ_ONLY",
            },
        )
        """
        response = self._raw_client.update(
            key,
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCustomerCustomAttributeDefinitionResponse:
        """
        Deletes a customer-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        Deleting a custom attribute definition also deletes the corresponding custom attribute from
        all customer profiles in the seller's Customer Directory.

        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCustomerCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.custom_attribute_definitions.delete(
            key="key",
        )
        """
        response = self._raw_client.delete(key, request_options=request_options)
        return response.data

    def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BatchUpsertCustomerCustomAttributesRequestCustomerCustomAttributeUpsertRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchUpsertCustomerCustomAttributesResponse:
        """
        Creates or updates [custom attributes](entity:CustomAttribute) for customer profiles as a bulk operation.

        Use this endpoint to set the value of one or more custom attributes for one or more customer profiles.
        A custom attribute is based on a custom attribute definition in a Square seller account, which is
        created using the [CreateCustomerCustomAttributeDefinition](api-endpoint:CustomerCustomAttributes-CreateCustomerCustomAttributeDefinition) endpoint.

        This `BulkUpsertCustomerCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides a customer ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BatchUpsertCustomerCustomAttributesRequestCustomerCustomAttributeUpsertRequestParams]
            A map containing 1 to 25 individual upsert requests. For each request, provide an
            arbitrary ID that is unique for this `BulkUpsertCustomerCustomAttributes` request and the
            information needed to create or update a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchUpsertCustomerCustomAttributesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.custom_attribute_definitions.batch_upsert(
            values={
                "id1": {
                    "customer_id": "N3NCVYY3WS27HF0HKANA3R9FP8",
                    "custom_attribute": {"key": "favoritemovie", "value": "Dune"},
                },
                "id2": {
                    "customer_id": "SY8EMWRNDN3TQDP2H4KS1QWMMM",
                    "custom_attribute": {"key": "ownsmovie", "value": False},
                },
                "id3": {
                    "customer_id": "SY8EMWRNDN3TQDP2H4KS1QWMMM",
                    "custom_attribute": {"key": "favoritemovie", "value": "Star Wars"},
                },
                "id4": {
                    "customer_id": "N3NCVYY3WS27HF0HKANA3R9FP8",
                    "custom_attribute": {
                        "key": "square:a0f1505a-2aa1-490d-91a8-8d31ff181808",
                        "value": "10.5",
                    },
                },
                "id5": {
                    "customer_id": "70548QG1HN43B05G0KCZ4MMC1G",
                    "custom_attribute": {
                        "key": "sq0ids-0evKIskIGaY45fCyNL66aw:backupemail",
                        "value": "fake-email@squareup.com",
                    },
                },
            },
        )
        """
        response = self._raw_client.batch_upsert(values=values, request_options=request_options)
        return response.data


class AsyncCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomAttributeDefinitionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomAttributeDefinitionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CustomAttributeDefinition]:
        """
        Lists the customer-related [custom attribute definitions](entity:CustomAttributeDefinition) that belong to a Square seller account.

        When all response pages are retrieved, the results include all custom attribute definitions
        that are visible to the requesting application, including those that are created by other
        applications and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that
        seller-defined custom attributes (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CustomAttributeDefinition]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.customers.custom_attribute_definitions.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/customers/custom-attribute-definitions",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCustomerCustomAttributeDefinitionsResponse,
                    construct_type(
                        type_=ListCustomerCustomAttributeDefinitionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attribute_definitions
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomerCustomAttributeDefinitionResponse:
        """
        Creates a customer-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to define a custom attribute that can be associated with customer profiles.

        A custom attribute definition specifies the `key`, `visibility`, `schema`, and other properties
        for a custom attribute. After the definition is created, you can call
        [UpsertCustomerCustomAttribute](api-endpoint:CustomerCustomAttributes-UpsertCustomerCustomAttribute) or
        [BulkUpsertCustomerCustomAttributes](api-endpoint:CustomerCustomAttributes-BulkUpsertCustomerCustomAttributes)
        to set the custom attribute for customer profiles in the seller's Customer Directory.

        Sellers can view all custom attributes in exported customer data, including those set to
        `VISIBILITY_HIDDEN`.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Specifying the schema](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attribute-definitions#specify-schema).
            - If provided, `name` must be unique (case-sensitive) across all visible customer-related custom attribute definitions for the seller.
            - All custom attributes are visible in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.custom_attribute_definitions.create(
                custom_attribute_definition={
                    "key": "favoritemovie",
                    "schema": {
                        "ref": "https://developer-production-s.squarecdn.com/schemas/v1/common.json#squareup.common.String"
                    },
                    "name": "Favorite Movie",
                    "description": "The favorite movie of the customer.",
                    "visibility": "VISIBILITY_HIDDEN",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomerCustomAttributeDefinitionResponse:
        """
        Retrieves a customer-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomerCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.custom_attribute_definitions.get(
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(key, version=version, request_options=request_options)
        return response.data

    async def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCustomerCustomAttributeDefinitionResponse:
        """
        Updates a customer-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.

        Use this endpoint to update the following fields: `name`, `description`, `visibility`, or the
        `schema` for a `Selection` data type.

        Only the definition owner can update a custom attribute definition. Note that sellers can view
        all custom attributes in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint
            supports sparse updates, so only new or changed fields need to be included in the request.
            Only the following fields can be updated:

            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCustomerCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.custom_attribute_definitions.update(
                key="key",
                custom_attribute_definition={
                    "description": "Update the description as desired.",
                    "visibility": "VISIBILITY_READ_ONLY",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            key,
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCustomerCustomAttributeDefinitionResponse:
        """
        Deletes a customer-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        Deleting a custom attribute definition also deletes the corresponding custom attribute from
        all customer profiles in the seller's Customer Directory.

        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCustomerCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.custom_attribute_definitions.delete(
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(key, request_options=request_options)
        return response.data

    async def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BatchUpsertCustomerCustomAttributesRequestCustomerCustomAttributeUpsertRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchUpsertCustomerCustomAttributesResponse:
        """
        Creates or updates [custom attributes](entity:CustomAttribute) for customer profiles as a bulk operation.

        Use this endpoint to set the value of one or more custom attributes for one or more customer profiles.
        A custom attribute is based on a custom attribute definition in a Square seller account, which is
        created using the [CreateCustomerCustomAttributeDefinition](api-endpoint:CustomerCustomAttributes-CreateCustomerCustomAttributeDefinition) endpoint.

        This `BulkUpsertCustomerCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides a customer ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BatchUpsertCustomerCustomAttributesRequestCustomerCustomAttributeUpsertRequestParams]
            A map containing 1 to 25 individual upsert requests. For each request, provide an
            arbitrary ID that is unique for this `BulkUpsertCustomerCustomAttributes` request and the
            information needed to create or update a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchUpsertCustomerCustomAttributesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.custom_attribute_definitions.batch_upsert(
                values={
                    "id1": {
                        "customer_id": "N3NCVYY3WS27HF0HKANA3R9FP8",
                        "custom_attribute": {"key": "favoritemovie", "value": "Dune"},
                    },
                    "id2": {
                        "customer_id": "SY8EMWRNDN3TQDP2H4KS1QWMMM",
                        "custom_attribute": {"key": "ownsmovie", "value": False},
                    },
                    "id3": {
                        "customer_id": "SY8EMWRNDN3TQDP2H4KS1QWMMM",
                        "custom_attribute": {
                            "key": "favoritemovie",
                            "value": "Star Wars",
                        },
                    },
                    "id4": {
                        "customer_id": "N3NCVYY3WS27HF0HKANA3R9FP8",
                        "custom_attribute": {
                            "key": "square:a0f1505a-2aa1-490d-91a8-8d31ff181808",
                            "value": "10.5",
                        },
                    },
                    "id5": {
                        "customer_id": "70548QG1HN43B05G0KCZ4MMC1G",
                        "custom_attribute": {
                            "key": "sq0ids-0evKIskIGaY45fCyNL66aw:backupemail",
                            "value": "fake-email@squareup.com",
                        },
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_upsert(values=values, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class WebhookSubscriptionParams(typing_extensions.TypedDict):
    """
    Represents the details of a webhook subscription, including notification URL,
    event types, and signature key.
    """

    id: typing_extensions.NotRequired[str]
    """
    A Square-generated unique ID for the subscription.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of this subscription.
    """

    enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the subscription is enabled (`true`) or not (`false`).
    """

    event_types: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The event types associated with this subscription.
    """

    notification_url: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The URL to which webhooks are sent.
    """

    api_version: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The API version of the subscription.
    This field is optional for `CreateWebhookSubscription`. 
    The value defaults to the API version used by the application.
    """

    signature_key: typing_extensions.NotRequired[str]
    """
    The Square-generated signature key used to validate the origin of the webhook event.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the subscription was created, in RFC 3339 format. For example, "2016-09-04T23:59:33.123Z".
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp of when the subscription was last updated, in RFC 3339 format.
    For example, "2016-09-04T23:59:33.123Z".
    """



# This file was auto-generated by Fern from our API Definition.

import typing

DisputeEvidenceType = typing.Union[
    typing.Literal[
        "GENERIC_EVIDENCE",
        "ONLINE_OR_APP_ACCESS_LOG",
        "AUTHORIZATION_DOCUMENTATION",
        "CANCELLATION_OR_REFUND_DOCUMENTATION",
        "CARDHOLDER_COMMUNICATION",
        "CARDHOLDER_INFORMATION",
        "PURCHASE_ACKNOWLEDGEMENT",
        "DUPLICATE_CHARGE_DOCUMENTATION",
        "PRODUCT_OR_SERVICE_DESCRIPTION",
        "RECEIPT",
        "SERVICE_RECEIVED_DOCUMENTATION",
        "PROOF_OF_DELIVERY_DOCUMENTATION",
        "RELATED_TRANSACTION_DOCUMENTATION",
        "REBUTTAL_EXPLANATION",
        "TRACKING_NUMBER",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .custom_field import CustomFieldParams
from .accepted_payment_methods import AcceptedPaymentMethodsParams
from .money import MoneyParams
from .shipping_fee import ShippingFeeParams


class CheckoutOptionsParams(typing_extensions.TypedDict):
    allow_tipping: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the payment allows tipping.
    """

    custom_fields: typing_extensions.NotRequired[typing.Optional[typing.Sequence[CustomFieldParams]]]
    """
    The custom fields requesting information from the buyer.
    """

    subscription_plan_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the subscription plan for the buyer to pay and subscribe.
    For more information, see [Subscription Plan Checkout](https://developer.squareup.com/docs/checkout-api/subscription-plan-checkout).
    """

    redirect_url: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The confirmation page URL to redirect the buyer to after Square processes the payment.
    """

    merchant_support_email: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The email address that buyers can use to contact the seller.
    """

    ask_for_shipping_address: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether to include the address fields in the payment form.
    """

    accepted_payment_methods: typing_extensions.NotRequired[AcceptedPaymentMethodsParams]
    """
    The methods allowed for buyers during checkout.
    """

    app_fee_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money that the developer is taking as a fee for facilitating the payment on behalf of the seller.
    
    The amount cannot be more than 90% of the total amount of the payment.
    
    The amount must be specified in the smallest denomination of the applicable currency (for example, US dollar amounts are specified in cents). For more information, see [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/common-data-types/working-with-monetary-amounts).
    
    The fee currency code must match the currency associated with the seller that is accepting the payment. The application must be from a developer account in the same country and using the same currency code as the seller. For more information about the application fee scenario, see [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).
    
    To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required. For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/collect-fees/additional-considerations#permissions).
    """

    shipping_fee: typing_extensions.NotRequired[ShippingFeeParams]
    """
    The fee associated with shipping to be applied to the `Order` as a service charge.
    """

    enable_coupon: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether to include the `Add coupon` section for the buyer to provide a Square marketing coupon in the payment form.
    """

    enable_loyalty: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether to include the `REWARDS` section for the buyer to opt in to loyalty, redeem rewards in the payment form, or both.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .measurement_unit import MeasurementUnitParams
import typing


class CatalogMeasurementUnitParams(typing_extensions.TypedDict):
    """
    Represents the unit used to measure a `CatalogItemVariation` and
    specifies the precision for decimal quantities.
    """

    measurement_unit: typing_extensions.NotRequired[MeasurementUnitParams]
    """
    Indicates the unit used to measure the quantity of a catalog item variation.
    """

    precision: typing_extensions.NotRequired[typing.Optional[int]]
    """
    An integer between 0 and 5 that represents the maximum number of
    positions allowed after the decimal in quantities measured with this unit.
    For example:
    
    - if the precision is 0, the quantity can be 1, 2, 3, etc.
    - if the precision is 1, the quantity can be 0.1, 0.2, etc.
    - if the precision is 2, the quantity can be 0.01, 0.12, etc.
    
    Default: 3
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.data_collection_options_input_type import DataCollectionOptionsInputType
import typing_extensions
from .collected_data import CollectedDataParams


class DataCollectionOptionsParams(typing_extensions.TypedDict):
    title: str
    """
    The title text to display in the data collection flow on the Terminal.
    """

    body: str
    """
    The body text to display under the title in the data collection screen flow on the
    Terminal.
    """

    input_type: DataCollectionOptionsInputType
    """
    Represents the type of the input text.
    See [InputType](#type-inputtype) for possible values
    """

    collected_data: typing_extensions.NotRequired[CollectedDataParams]
    """
    The buyer’s input text from the data collection screen.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderReturnLineItemModifier(UncheckedBaseModel):
    """
    A line item modifier being returned.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID that identifies the return modifier only within this order.
    """

    source_modifier_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The modifier `uid` from the order's line item that contains the
    original sale of this line item modifier.
    """

    catalog_object_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The catalog object ID referencing [CatalogModifier](entity:CatalogModifier).
    """

    catalog_version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version of the catalog object that this line item modifier references.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the item modifier.
    """

    base_price_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The base price for the modifier.
    
    `base_price_money` is required for ad hoc modifiers.
    If both `catalog_object_id` and `base_price_money` are set, `base_price_money` overrides the predefined [CatalogModifier](entity:CatalogModifier) price.
    """

    total_price_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The total price of the item modifier for its line item.
    This is the modifier's `base_price_money` multiplied by the line item's quantity.
    """

    quantity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The quantity of the line item modifier. The modifier quantity can be 0 or more.
    For example, suppose a restaurant offers a cheeseburger on the menu. When a buyer orders
    this item, the restaurant records the purchase by creating an `Order` object with a line item
    for a burger. The line item includes a line item modifier: the name is cheese and the quantity
    is 1. The buyer has the option to order extra cheese (or no cheese). If the buyer chooses
    the extra cheese option, the modifier quantity increases to 2. If the buyer does not want
    any cheese, the modifier quantity is set to 0.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class EventTypeMetadataParams(typing_extensions.TypedDict):
    """
    Contains the metadata of a webhook event type.
    """

    event_type: typing_extensions.NotRequired[str]
    """
    The event type.
    """

    api_version_introduced: typing_extensions.NotRequired[str]
    """
    The API version at which the event type was introduced.
    """

    release_status: typing_extensions.NotRequired[str]
    """
    The release status of the event type.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .merchant import MerchantParams


class ListMerchantsResponseParams(typing_extensions.TypedDict):
    """
    The response object returned by the [ListMerchant](api-endpoint:Merchants-ListMerchants) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    merchant: typing_extensions.NotRequired[typing.Sequence[MerchantParams]]
    """
    The requested `Merchant` entities.
    """

    cursor: typing_extensions.NotRequired[int]
    """
    If the  response is truncated, the cursor to use in next  request to fetch next set of objects.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .inventory_adjustment import InventoryAdjustment
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GetInventoryAdjustmentResponse(UncheckedBaseModel):
    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    adjustment: typing.Optional[InventoryAdjustment] = pydantic.Field(default=None)
    """
    The requested [InventoryAdjustment](entity:InventoryAdjustment).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawSitesClient
import typing
from ..core.request_options import RequestOptions
from ..types.list_sites_response import ListSitesResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawSitesClient


class SitesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSitesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSitesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSitesClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListSitesResponse:
        """
        Lists the Square Online sites that belong to a seller. Sites are listed in descending order by the `created_at` date.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSitesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.sites.list()
        """
        response = self._raw_client.list(request_options=request_options)
        return response.data


class AsyncSitesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSitesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSitesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSitesClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListSitesResponse:
        """
        Lists the Square Online sites that belong to a seller. Sites are listed in descending order by the `created_at` date.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSitesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.sites.list()


        asyncio.run(main())
        """
        response = await self._raw_client.list(request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing

MeasurementUnitVolume = typing.Union[
    typing.Literal[
        "GENERIC_FLUID_OUNCE",
        "GENERIC_SHOT",
        "GENERIC_CUP",
        "GENERIC_PINT",
        "GENERIC_QUART",
        "GENERIC_GALLON",
        "IMPERIAL_CUBIC_INCH",
        "IMPERIAL_CUBIC_FOOT",
        "IMPERIAL_CUBIC_YARD",
        "METRIC_MILLILITER",
        "METRIC_LITER",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogObjectReferenceParams(typing_extensions.TypedDict):
    """
    A reference to a Catalog object at a specific version. In general this is
    used as an entry point into a graph of catalog objects, where the objects exist
    at a specific version.
    """

    object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the referenced object.
    """

    catalog_version: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The version of the object.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .payment import PaymentParams


class CompletePaymentResponseParams(typing_extensions.TypedDict):
    """
    Defines the response returned by[CompletePayment](api-endpoint:Payments-CompletePayment).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    payment: typing_extensions.NotRequired[PaymentParams]
    """
    The successfully completed payment.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .get_team_member_booking_profile_response import GetTeamMemberBookingProfileResponse
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkRetrieveTeamMemberBookingProfilesResponse(UncheckedBaseModel):
    """
    Response payload for the [BulkRetrieveTeamMemberBookingProfiles](api-endpoint:Bookings-BulkRetrieveTeamMemberBookingProfiles) endpoint.
    """

    team_member_booking_profiles: typing.Optional[typing.Dict[str, GetTeamMemberBookingProfileResponse]] = (
        pydantic.Field(default=None)
    )
    """
    The returned team members' booking profiles, as a map with `team_member_id` as the key and [TeamMemberBookingProfile](entity:TeamMemberBookingProfile) the value.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .catalog_object import CatalogObjectParams


class SearchCatalogObjectsResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request. If unset, this is the final response.
    See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.
    """

    objects: typing_extensions.NotRequired[typing.Sequence[CatalogObjectParams]]
    """
    The CatalogObjects returned.
    """

    related_objects: typing_extensions.NotRequired[typing.Sequence[CatalogObjectParams]]
    """
    A list of CatalogObjects referenced by the objects in the `objects` field.
    """

    latest_time: typing_extensions.NotRequired[str]
    """
    When the associated product catalog was last updated. Will
    match the value for `end_time` or `cursor` if either field is included in the `SearchCatalog` request.
    """
